@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_attr.c,v 1.131 2022/09/01 13:19:11 claudio Exp $ */
+/*	$OpenBSD: rde_attr.c,v 1.133 2023/06/12 12:10:17 claudio Exp $ */
 
 /*
  * Copyright (c) 2004 Claudio Jeker <claudio@openbsd.org>
@@ -85,7 +85,7 @@ attr_writebuf(struct ibuf *buf, uint8_t flags, uint8_t type, void *data,
 
 	if (ibuf_add(buf, hdr, flags & ATTR_EXTLEN ? 4 : 3) == -1)
 		return (-1);
-	if (data && ibuf_add(buf, data, data_len) == -1)
+	if (data != NULL && ibuf_add(buf, data, data_len) == -1)
 		return (-1);
 	return (0);
 }
@@ -516,18 +516,6 @@ aspath_merge(struct rde_aspath *a, struct attr *attr)
 	attr_free(a, attr);
 }
 
-u_char *
-aspath_dump(struct aspath *aspath)
-{
-	return (aspath->data);
-}
-
-uint16_t
-aspath_length(struct aspath *aspath)
-{
-	return (aspath->len);
-}
-
 uint32_t
 aspath_neighbor(struct aspath *aspath)
 {
@@ -542,12 +530,6 @@ aspath_neighbor(struct aspath *aspath)
 	return (aspath_extract(aspath->data, 0));
 }
 
-uint32_t
-aspath_origin(struct aspath *aspath)
-{
-	return aspath->source_as;
-}
-
 static uint16_t
 aspath_count(const void *data, uint16_t len)
 {
