@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_decide.c,v 1.98 2022/09/23 15:49:20 claudio Exp $ */
+/*	$OpenBSD: rde_decide.c,v 1.101 2023/03/13 16:52:42 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org>
@@ -487,7 +487,8 @@ prefix_eligible(struct prefix *p)
 	struct rde_aspath *asp = prefix_aspath(p);
 
 	/* The aspath needs to be loop and error free */
-	if (asp == NULL || asp->flags & (F_ATTR_LOOP|F_ATTR_PARSE_ERR))
+	if (asp == NULL ||
+	    asp->flags & (F_ATTR_LOOP|F_ATTR_OTC_LEAK|F_ATTR_PARSE_ERR))
 		return 0;
 
 	/* The nexthop must be valid. */
@@ -556,10 +557,9 @@ prefix_evaluate(struct rib_entry *re, struct prefix *new, struct prefix *old)
 		 * but remember that newbest may be NULL aka ineligible.
 		 * Additional decision may be made by the called functions.
 		 */
-		rde_generate_updates(rib, newbest, oldbest, new, old,
-		    EVAL_DEFAULT);
 		if ((rib->flags & F_RIB_NOFIB) == 0)
 			rde_send_kroute(rib, newbest, oldbest);
+		rde_generate_updates(re, new, old, EVAL_DEFAULT);
 		return;
 	}
 
@@ -570,8 +570,7 @@ prefix_evaluate(struct rib_entry *re, struct prefix *new, struct prefix *old)
 	 */
 	if (rde_evaluate_all())
 		if ((new != NULL && prefix_eligible(new)) || old != NULL)
-			rde_generate_updates(rib, newbest, NULL, new, old,
-			    EVAL_ALL);
+			rde_generate_updates(re, new, old, EVAL_ALL);
 }
 
 void
@@ -630,9 +629,9 @@ prefix_evaluate_nexthop(struct prefix *p, enum nexthop_state state,
 		 * but remember that newbest may be NULL aka ineligible.
 		 * Additional decision may be made by the called functions.
 		 */
-		rde_generate_updates(rib, newbest, oldbest, p, p, EVAL_DEFAULT);
 		if ((rib->flags & F_RIB_NOFIB) == 0)
 			rde_send_kroute(rib, newbest, oldbest);
+		rde_generate_updates(re, p, p, EVAL_DEFAULT);
 		return;
 	}
 
@@ -642,5 +641,5 @@ prefix_evaluate_nexthop(struct prefix *p, enum nexthop_state state,
 	 * rde_generate_updates() will then take care of distribution.
 	 */
 	if (rde_evaluate_all())
-		rde_generate_updates(rib, newbest, NULL, p, p, EVAL_ALL);
+		rde_generate_updates(re, p, p, EVAL_ALL);
 }
