@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_update.c,v 1.148 2022/09/23 15:49:20 claudio Exp $ */
+/*	$OpenBSD: rde_update.c,v 1.162 2023/04/19 08:30:37 claudio Exp $ */
 
 /*
  * Copyright (c) 2004 Claudio Jeker <claudio@openbsd.org>
@@ -28,6 +28,13 @@
 #include "rde.h"
 #include "log.h"
 
+enum up_state {
+	UP_OK,
+	UP_ERR_LIMIT,
+	UP_FILTERED,
+	UP_EXCLUDED,
+};
+
 static struct community	comm_no_advertise = {
 	.flags = COMMUNITY_TYPE_BASIC,
 	.data1 = COMMUNITY_WELLKNOWN,
@@ -59,7 +66,7 @@ up_test_update(struct rde_peer *peer, struct prefix *p)
 
 	if (asp == NULL || asp->flags & F_ATTR_PARSE_ERR)
 		fatalx("try to send out a botched path");
-	if (asp->flags & F_ATTR_LOOP)
+	if (asp->flags & (F_ATTR_LOOP | F_ATTR_OTC_LEAK))
 		fatalx("try to send out a looped path");
 
 	if (peer == frompeer)
@@ -97,27 +104,28 @@ up_test_update(struct rde_peer *peer, struct prefix *p)
 
 /* RFC9234 open policy handling */
 static int
-up_enforce_open_policy(struct rde_peer *peer, struct filterstate *state)
+up_enforce_open_policy(struct rde_peer *peer, struct filterstate *state,
+    uint8_t aid)
 {
-	uint8_t role;
-
-	if (!peer_has_open_policy(peer, &role))
+	/* only for IPv4 and IPv6 unicast */
+	if (aid != AID_INET && aid != AID_INET6)
 		return 0;
 
 	/*
 	 * do not propagate (consider it filtered) if OTC is present and
-	 * neighbor role is peer, provider or rs.
+	 * local role is peer, customer or rs-client.
 	 */
-	if (role == CAPA_ROLE_PEER || role == CAPA_ROLE_PROVIDER ||
-	    role == CAPA_ROLE_RS)
+	if (peer->role == ROLE_PEER || peer->role == ROLE_CUSTOMER ||
+	    peer->role == ROLE_RS_CLIENT)
 		if (state->aspath.flags & F_ATTR_OTC)
-			return (1);
+			return 1;
 
 	/*
-	 * add OTC attribute if not present for peers, customers and rs-clients.
+	 * add OTC attribute if not present towards peers, customers and
+	 * rs-clients (local roles peer, provider, rs).
 	 */
-	if (role == CAPA_ROLE_PEER || role == CAPA_ROLE_CUSTOMER ||
-	    role == CAPA_ROLE_RS_CLIENT)
+	if (peer->role == ROLE_PEER || peer->role == ROLE_PROVIDER ||
+	    peer->role == ROLE_RS)
 		if ((state->aspath.flags & F_ATTR_OTC) == 0) {
 			uint32_t tmp;
 
@@ -133,97 +141,97 @@ up_enforce_open_policy(struct rde_peer *peer, struct filterstate *state)
 	return 0;
 }
 
-void
-up_generate_updates(struct filter_head *rules, struct rde_peer *peer,
-    struct prefix *new, struct prefix *old)
+/*
+ * Process a single prefix by passing it through the various filter stages
+ * and if not filtered out update the Adj-RIB-Out. Returns:
+ * - UP_OK if prefix was added
+ * - UP_ERR_LIMIT if the peer outbound prefix limit was reached
+ * - UP_FILTERED if prefix was filtered out
+ * - UP_EXCLUDED if prefix was excluded because of up_test_update()
+ */
+static enum up_state
+up_process_prefix(struct rde_peer *peer, struct prefix *new, struct prefix *p)
 {
-	struct filterstate	state;
-	struct bgpd_addr	addr;
-	struct prefix		*p;
-	int			need_withdraw;
-	uint8_t			prefixlen;
+	struct filterstate state;
+	struct bgpd_addr addr;
+	int excluded = 0;
 
-	if (new == NULL) {
-		pt_getaddr(old->pt, &addr);
-		prefixlen = old->pt->prefixlen;
-	} else {
-		pt_getaddr(new->pt, &addr);
-		prefixlen = new->pt->prefixlen;
-	}
+	/*
+	 * up_test_update() needs to run before the output filters
+	 * else the well known communities won't work properly.
+	 * The output filters would not be able to add well known
+	 * communities.
+	 */
+	if (!up_test_update(peer, new))
+		excluded = 1;
 
-	p = prefix_adjout_lookup(peer, &addr, prefixlen);
-
-	while (new != NULL) {
-		need_withdraw = 0;
-		/*
-		 * up_test_update() needs to run before the output filters
-		 * else the well known communities won't work properly.
-		 * The output filters would not be able to add well known
-		 * communities.
-		 */
-		if (!up_test_update(peer, new))
-			need_withdraw = 1;
-
-		/*
-		 * if 'rde evaluate all' is set for this peer then
-		 * delay the the withdraw because of up_test_update().
-		 * The filters may actually skip this prefix and so this
-		 * decision needs to be delayed.
-		 * For the default mode we can just give up here and
-		 * skip the filters.
-		 */
-		if (need_withdraw &&
-		    !(peer->flags & PEERFLAG_EVALUATE_ALL))
-			break;
-
-		rde_filterstate_prep(&state, prefix_aspath(new),
-		    prefix_communities(new), prefix_nexthop(new),
-		    prefix_nhflags(new));
-		if (rde_filter(rules, peer, prefix_peer(new), &addr,
-		    prefixlen, prefix_vstate(new), &state) == ACTION_DENY) {
-			rde_filterstate_clean(&state);
-			if (peer->flags & PEERFLAG_EVALUATE_ALL) {
-				new = TAILQ_NEXT(new, entry.list.rib);
-				if (new != NULL && prefix_eligible(new))
-					continue;
-			}
-			break;
-		}
-
-		if (up_enforce_open_policy(peer, &state)) {
-			rde_filterstate_clean(&state);
-			if (peer->flags & PEERFLAG_EVALUATE_ALL) {
-				new = TAILQ_NEXT(new, entry.list.rib);
-				if (new != NULL && prefix_eligible(new))
-					continue;
-			}
-			break;
-		}
-
-		/* check if this was actually a withdraw */
-		if (need_withdraw)
-			break;
-
-		/* from here on we know this is an update */
-
-		up_prep_adjout(peer, &state, addr.aid);
-		prefix_adjout_update(p, peer, &state, &addr,
-		    new->pt->prefixlen, new->path_id_tx, prefix_vstate(new));
+	rde_filterstate_prep(&state, new);
+	pt_getaddr(new->pt, &addr);
+	if (rde_filter(peer->out_rules, peer, prefix_peer(new), &addr,
+	    new->pt->prefixlen, &state) == ACTION_DENY) {
 		rde_filterstate_clean(&state);
-
-		/* max prefix checker outbound */
-		if (peer->conf.max_out_prefix &&
-		    peer->prefix_out_cnt > peer->conf.max_out_prefix) {
-			log_peer_warnx(&peer->conf,
-			    "outbound prefix limit reached (>%u/%u)",
-			    peer->prefix_out_cnt, peer->conf.max_out_prefix);
-			rde_update_err(peer, ERR_CEASE,
-			    ERR_CEASE_MAX_SENT_PREFIX, NULL, 0);
-		}
-
-		return;
+		return UP_FILTERED;
 	}
 
+	/* Open Policy Check: acts like an output filter */
+	if (up_enforce_open_policy(peer, &state, new->pt->aid)) {
+		rde_filterstate_clean(&state);
+		return UP_FILTERED;
+	}
+
+	if (excluded) {
+		rde_filterstate_clean(&state);
+		return UP_EXCLUDED;
+	}
+
+	/* from here on we know this is an update */
+	if (p == (void *)-1)
+		p = prefix_adjout_get(peer, new->path_id_tx, new->pt);
+
+	up_prep_adjout(peer, &state, new->pt->aid);
+	prefix_adjout_update(p, peer, &state, new->pt, new->path_id_tx);
+	rde_filterstate_clean(&state);
+
+	/* max prefix checker outbound */
+	if (peer->conf.max_out_prefix &&
+	    peer->stats.prefix_out_cnt > peer->conf.max_out_prefix) {
+		log_peer_warnx(&peer->conf,
+		    "outbound prefix limit reached (>%u/%u)",
+		    peer->stats.prefix_out_cnt, peer->conf.max_out_prefix);
+		rde_update_err(peer, ERR_CEASE,
+		    ERR_CEASE_MAX_SENT_PREFIX, NULL, 0);
+		return UP_ERR_LIMIT;
+	}
+
+	return UP_OK;
+}
+
+void
+up_generate_updates(struct rde_peer *peer, struct rib_entry *re)
+{
+	struct prefix		*new, *p;
+
+	p = prefix_adjout_first(peer, re->prefix);
+
+	new = prefix_best(re);
+	while (new != NULL) {
+		switch (up_process_prefix(peer, new, p)) {
+		case UP_OK:
+		case UP_ERR_LIMIT:
+			return;
+		case UP_FILTERED:
+			if (peer->flags & PEERFLAG_EVALUATE_ALL) {
+				new = TAILQ_NEXT(new, entry.list.rib);
+				if (new != NULL && prefix_eligible(new))
+					continue;
+			}
+			goto done;
+		case UP_EXCLUDED:
+			goto done;
+		}
+	}
+
+done:
 	/* withdraw prefix */
 	if (p != NULL)
 		prefix_adjout_withdraw(p);
@@ -237,36 +245,21 @@ up_generate_updates(struct filter_head *rules, struct rde_peer *peer,
  * less churn is needed.
  */
 void
-up_generate_addpath(struct filter_head *rules, struct rde_peer *peer,
-    struct prefix *new, struct prefix *old)
+up_generate_addpath(struct rde_peer *peer, struct rib_entry *re)
 {
-	struct filterstate	state;
-	struct bgpd_addr	addr;
-	struct prefix		*head, *p;
-	uint8_t			prefixlen;
+	struct prefix		*head, *new, *p;
 	int			maxpaths = 0, extrapaths = 0, extra;
 	int			checkmode = 1;
 
-	if (new == NULL) {
-		pt_getaddr(old->pt, &addr);
-		prefixlen = old->pt->prefixlen;
-	} else {
-		pt_getaddr(new->pt, &addr);
-		prefixlen = new->pt->prefixlen;
-	}
-
-	head = prefix_adjout_lookup(peer, &addr, prefixlen);
+	head = prefix_adjout_first(peer, re->prefix);
 
 	/* mark all paths as stale */
 	for (p = head; p != NULL; p = prefix_adjout_next(peer, p))
 		p->flags |= PREFIX_FLAG_STALE;
 
 	/* update paths */
-	for ( ; new != NULL; new = TAILQ_NEXT(new, entry.list.rib)) {
-		/* since list is sorted, stop at first invalid prefix */
-		if (!prefix_eligible(new))
-			break;
-
+	new = prefix_best(re);
+	while (new != NULL) {
 		/* check limits and stop when a limit is reached */
 		if (peer->eval.maxpaths != 0 &&
 		    maxpaths >= peer->eval.maxpaths)
@@ -308,50 +301,23 @@ up_generate_addpath(struct filter_head *rules, struct rde_peer *peer,
 			}
 		}
 
-		/*
-		 * up_test_update() needs to run before the output filters
-		 * else the well known communities won't work properly.
-		 * The output filters would not be able to add well known
-		 * communities.
-		 */
-		if (!up_test_update(peer, new))
-			continue;
-
-		rde_filterstate_prep(&state, prefix_aspath(new),
-		    prefix_communities(new), prefix_nexthop(new),
-		    prefix_nhflags(new));
-		if (rde_filter(rules, peer, prefix_peer(new), &addr,
-		    prefixlen, prefix_vstate(new), &state) == ACTION_DENY) {
-			rde_filterstate_clean(&state);
-			continue;
+		switch (up_process_prefix(peer, new, (void *)-1)) {
+		case UP_OK:
+			maxpaths++;
+			extrapaths += extra;
+			break;
+		case UP_FILTERED:
+		case UP_EXCLUDED:
+			break;
+		case UP_ERR_LIMIT:
+			/* just give up */
+			return;
 		}
 
-		if (up_enforce_open_policy(peer, &state)) {
-			rde_filterstate_clean(&state);
-			continue;
-		}
-
-		/* from here on we know this is an update */
-		maxpaths++;
-		extrapaths += extra;
-
-		p = prefix_adjout_get(peer, new->path_id_tx, &addr,
-		    new->pt->prefixlen);
-
-		up_prep_adjout(peer, &state, addr.aid);
-		prefix_adjout_update(p, peer, &state, &addr,
-		    new->pt->prefixlen, new->path_id_tx, prefix_vstate(new));
-		rde_filterstate_clean(&state);
-
-		/* max prefix checker outbound */
-		if (peer->conf.max_out_prefix &&
-		    peer->prefix_out_cnt > peer->conf.max_out_prefix) {
-			log_peer_warnx(&peer->conf,
-			    "outbound prefix limit reached (>%u/%u)",
-			    peer->prefix_out_cnt, peer->conf.max_out_prefix);
-			rde_update_err(peer, ERR_CEASE,
-			    ERR_CEASE_MAX_SENT_PREFIX, NULL, 0);
-		}
+		/* only allow valid prefixes */
+		new = TAILQ_NEXT(new, entry.list.rib);
+		if (new == NULL || !prefix_eligible(new))
+			break;
 	}
 
 	/* withdraw stale paths */
@@ -366,13 +332,10 @@ up_generate_addpath(struct filter_head *rules, struct rde_peer *peer,
  * are distributed just remove old and add new.
  */ 
 void
-up_generate_addpath_all(struct filter_head *rules, struct rde_peer *peer,
-    struct prefix *best, struct prefix *new, struct prefix *old)
+up_generate_addpath_all(struct rde_peer *peer, struct rib_entry *re,
+    struct prefix *new, struct prefix *old)
 {
-	struct filterstate	state;
-	struct bgpd_addr	addr;
-	struct prefix		*p, *next, *head = NULL;
-	uint8_t			prefixlen;
+	struct prefix		*p, *head = NULL;
 	int			all = 0;
 
 	/*
@@ -381,82 +344,40 @@ up_generate_addpath_all(struct filter_head *rules, struct rde_peer *peer,
 	 */
 	if (old == NULL && new == NULL) {
 		/* mark all paths as stale */
-		pt_getaddr(best->pt, &addr);
-		prefixlen = best->pt->prefixlen;
-
-		head = prefix_adjout_lookup(peer, &addr, prefixlen);
+		head = prefix_adjout_first(peer, re->prefix);
 		for (p = head; p != NULL; p = prefix_adjout_next(peer, p))
 			p->flags |= PREFIX_FLAG_STALE;
 
-		new = best;
+		new = prefix_best(re);
 		all = 1;
 	}
 
 	if (old != NULL) {
 		/* withdraw stale paths */
-		pt_getaddr(old->pt, &addr);
-		p = prefix_adjout_get(peer, old->path_id_tx, &addr,
-		    old->pt->prefixlen);
+		p = prefix_adjout_get(peer, old->path_id_tx, old->pt);
 		if (p != NULL)
 			prefix_adjout_withdraw(p);
 	}
 
-	if (new != NULL) {
-		pt_getaddr(new->pt, &addr);
-		prefixlen = new->pt->prefixlen;
-	}
-
 	/* add new path (or multiple if all is set) */
-	for (; new != NULL; new = next) {
-		if (all)
-			next = TAILQ_NEXT(new, entry.list.rib);
-		else
-			next = NULL;
+	while (new != NULL) {
+		switch (up_process_prefix(peer, new, (void *)-1)) {
+		case UP_OK:
+		case UP_FILTERED:
+		case UP_EXCLUDED:
+			break;
+		case UP_ERR_LIMIT:
+			/* just give up */
+			return;
+		}
 
-		/* only allow valid prefixes */
-		if (!prefix_eligible(new))
+		if (!all)
 			break;
 
-		/*
-		 * up_test_update() needs to run before the output filters
-		 * else the well known communities won't work properly.
-		 * The output filters would not be able to add well known
-		 * communities.
-		 */
-		if (!up_test_update(peer, new))
-			continue;
-
-		rde_filterstate_prep(&state, prefix_aspath(new),
-		    prefix_communities(new), prefix_nexthop(new),
-		    prefix_nhflags(new));
-		if (rde_filter(rules, peer, prefix_peer(new), &addr,
-		    prefixlen, prefix_vstate(new), &state) == ACTION_DENY) {
-			rde_filterstate_clean(&state);
-			continue;
-		}
-
-		if (up_enforce_open_policy(peer, &state)) {
-			rde_filterstate_clean(&state);
-			continue;
-		}
-
-		/* from here on we know this is an update */
-		p = prefix_adjout_get(peer, new->path_id_tx, &addr, prefixlen);
-
-		up_prep_adjout(peer, &state, addr.aid);
-		prefix_adjout_update(p, peer, &state, &addr,
-		    prefixlen, new->path_id_tx, prefix_vstate(new));
-		rde_filterstate_clean(&state);
-
-		/* max prefix checker outbound */
-		if (peer->conf.max_out_prefix &&
-		    peer->prefix_out_cnt > peer->conf.max_out_prefix) {
-			log_peer_warnx(&peer->conf,
-			    "outbound prefix limit reached (>%u/%u)",
-			    peer->prefix_out_cnt, peer->conf.max_out_prefix);
-			rde_update_err(peer, ERR_CEASE,
-			    ERR_CEASE_MAX_SENT_PREFIX, NULL, 0);
-		}
+		/* only allow valid prefixes */
+		new = TAILQ_NEXT(new, entry.list.rib);
+		if (new == NULL || !prefix_eligible(new))
+			break;
 	}
 
 	if (all) {
@@ -468,28 +389,25 @@ up_generate_addpath_all(struct filter_head *rules, struct rde_peer *peer,
 	}
 }
 
-struct rib_entry *rib_add(struct rib *, struct bgpd_addr *, int);
-void rib_remove(struct rib_entry *);
-int rib_empty(struct rib_entry *);
-
 /* send a default route to the specified peer */
 void
-up_generate_default(struct filter_head *rules, struct rde_peer *peer,
-    uint8_t aid)
+up_generate_default(struct rde_peer *peer, uint8_t aid)
 {
 	extern struct rde_peer	*peerself;
 	struct filterstate	 state;
 	struct rde_aspath	*asp;
 	struct prefix		*p;
+	struct pt_entry		*pte;
 	struct bgpd_addr	 addr;
 
 	if (peer->capa.mp[aid] == 0)
 		return;
 
-	rde_filterstate_prep(&state, NULL, NULL, NULL, 0);
+	rde_filterstate_init(&state);
 	asp = &state.aspath;
 	asp->aspath = aspath_get(NULL, 0);
 	asp->origin = ORIGIN_IGP;
+	rde_filterstate_set_vstate(&state, ROA_NOTFOUND, ASPA_NEVER_KNOWN);
 	/* the other default values are OK, nexthop is once again NULL */
 
 	/*
@@ -503,22 +421,26 @@ up_generate_default(struct filter_head *rules, struct rde_peer *peer,
 	p = prefix_adjout_lookup(peer, &addr, 0);
 
 	/* outbound filter as usual */
-	if (rde_filter(rules, peer, peerself, &addr, 0, ROA_NOTFOUND,
-	    &state) == ACTION_DENY) {
+	if (rde_filter(peer->out_rules, peer, peerself, &addr, 0, &state) ==
+	    ACTION_DENY) {
 		rde_filterstate_clean(&state);
 		return;
 	}
 
 	up_prep_adjout(peer, &state, addr.aid);
-	prefix_adjout_update(p, peer, &state, &addr, 0, 0, ROA_NOTFOUND);
+	/* can't use pt_fill here since prefix_adjout_update keeps a ref */
+	pte = pt_get(&addr, 0);
+	if (pte == NULL)
+		pte = pt_add(&addr, 0);
+	prefix_adjout_update(p, peer, &state, pte, 0);
 	rde_filterstate_clean(&state);
 
 	/* max prefix checker outbound */
 	if (peer->conf.max_out_prefix &&
-	    peer->prefix_out_cnt > peer->conf.max_out_prefix) {
+	    peer->stats.prefix_out_cnt > peer->conf.max_out_prefix) {
 		log_peer_warnx(&peer->conf,
 		    "outbound prefix limit reached (>%u/%u)",
-		    peer->prefix_out_cnt, peer->conf.max_out_prefix);
+		    peer->stats.prefix_out_cnt, peer->conf.max_out_prefix);
 		rde_update_err(peer, ERR_CEASE,
 		    ERR_CEASE_MAX_SENT_PREFIX, NULL, 0);
 	}
@@ -538,6 +460,10 @@ up_get_nexthop(struct rde_peer *peer, struct filterstate *state, uint8_t aid)
 	case AID_VPN_IPv6:
 		peer_local = &peer->local_v6_addr;
 		break;
+	case AID_FLOWSPECv4:
+	case AID_FLOWSPECv6:
+		/* flowspec has no nexthop */
+		return (NULL);
 	default:
 		fatalx("%s, bad AID %s", __func__, aid2str(aid));
 	}
@@ -608,7 +534,7 @@ static void
 up_prep_adjout(struct rde_peer *peer, struct filterstate *state, uint8_t aid)
 {
 	struct bgpd_addr *nexthop;
-	struct nexthop *nh;
+	struct nexthop *nh = NULL;
 	u_char *np;
 	uint16_t nl;
 
@@ -623,7 +549,8 @@ up_prep_adjout(struct rde_peer *peer, struct filterstate *state, uint8_t aid)
 
 	/* update nexthop */
 	nexthop = up_get_nexthop(peer, state, aid);
-	nh = nexthop_get(nexthop);
+	if (nexthop != NULL)
+		nh = nexthop_get(nexthop);
 	nexthop_unref(state->nexthop);
 	state->nexthop = nh;
 	state->nhflags = 0;
@@ -899,7 +826,6 @@ up_dump_prefix(u_char *buf, int len, struct prefix_tree *prefix_head,
     struct rde_peer *peer, int withdraw)
 {
 	struct prefix	*p, *np;
-	struct bgpd_addr addr;
 	uint32_t	 pathid;
 	int		 r, wpos = 0, done = 0;
 
@@ -911,9 +837,8 @@ up_dump_prefix(u_char *buf, int len, struct prefix_tree *prefix_head,
 			memcpy(buf + wpos, &pathid, sizeof(pathid));
 			wpos += sizeof(pathid);
 		}
-		pt_getaddr(p->pt, &addr);
-		if ((r = prefix_write(buf + wpos, len - wpos,
-		    &addr, p->pt->prefixlen, withdraw)) == -1) {
+		if ((r = pt_write(buf + wpos, len - wpos, p->pt,
+		    withdraw)) == -1) {
 			if (peer_has_add_path(peer, p->pt->aid, CAPA_AP_SEND))
 				wpos -= sizeof(pathid);
 			break;
@@ -932,13 +857,13 @@ up_dump_prefix(u_char *buf, int len, struct prefix_tree *prefix_head,
 		if (withdraw) {
 			/* prefix no longer needed, remove it */
 			prefix_adjout_destroy(p);
-			peer->prefix_sent_withdraw++;
+			peer->stats.prefix_sent_withdraw++;
 		} else {
 			/* prefix still in Adj-RIB-Out, keep it */
 			RB_REMOVE(prefix_tree, prefix_head, p);
 			p->flags &= ~PREFIX_FLAG_UPDATE;
-			peer->up_nlricnt--;
-			peer->prefix_sent_update++;
+			peer->stats.pending_update--;
+			peer->stats.prefix_sent_update++;
 		}
 
 		if (done)
@@ -1028,9 +953,7 @@ up_dump_attrnlri(u_char *buf, int len, struct rde_peer *peer)
 	if (p == NULL)
 		goto done;
 
-	rde_filterstate_prep(&state, prefix_aspath(p), prefix_communities(p),
-	    prefix_nexthop(p), prefix_nhflags(p));
-
+	rde_filterstate_prep(&state, p);
 	r = up_generate_attr(buf + 2, len - 2, peer, &state, AID_INET);
 	rde_filterstate_clean(&state);
 	if (r == -1) {
@@ -1130,6 +1053,19 @@ up_generate_mp_reach(u_char *buf, int len, struct rde_peer *peer,
 		nexthop = &state->nexthop->exit_nexthop;
 		memcpy(attrbuf, &nexthop->v6, sizeof(struct in6_addr));
 		break;
+	case AID_FLOWSPECv4:
+	case AID_FLOWSPECv6:
+		attrlen = 5; /* AFI + SAFI + NH LEN + NH + Reserved */
+		if (len < wpos + attrlen)
+			return (-1);
+		wpos += attrlen;
+		if (aid2afi(aid, &tmp, &attrbuf[2]))
+			fatalx("up_generate_mp_reachi: bad AID");
+		tmp = htons(tmp);
+		memcpy(attrbuf, &tmp, sizeof(tmp));
+		attrbuf[3] = 0;	/* nh len MUST be 0 */
+		attrbuf[4] = 0;	/* Reserved must be 0 */
+		break;
 	default:
 		fatalx("up_generate_mp_reach: unknown AID");
 	}
@@ -1167,8 +1103,7 @@ up_dump_mp_reach(u_char *buf, int len, struct rde_peer *peer, uint8_t aid)
 
 	wpos = 4;	/* reserve space for length fields */
 
-	rde_filterstate_prep(&state, prefix_aspath(p), prefix_communities(p),
-	    prefix_nexthop(p), prefix_nhflags(p));
+	rde_filterstate_prep(&state, p);
 
 	/* write regular path attributes */
 	r = up_generate_attr(buf + wpos, len - wpos, peer, &state, aid);
