@@ -1,4 +1,4 @@
-/*	$OpenBSD: rtr_proto.c,v 1.7 2022/07/11 16:47:27 claudio Exp $ */
+/*	$OpenBSD: rtr_proto.c,v 1.16 2023/03/28 12:15:23 claudio Exp $ */
 
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -35,10 +35,12 @@ struct rtr_header {
 	uint32_t	length;
 };
 
+#define RTR_MAX_VERSION		2
 #define RTR_MAX_LEN		2048
 #define RTR_DEFAULT_REFRESH	3600
 #define RTR_DEFAULT_RETRY	600
 #define RTR_DEFAULT_EXPIRE	7200
+#define RTR_DEFAULT_ACTIVE	60
 
 enum rtr_pdu_type {
 	SERIAL_NOTIFY = 0,
@@ -51,6 +53,7 @@ enum rtr_pdu_type {
 	CACHE_RESET = 8,
 	ROUTER_KEY = 9,
 	ERROR_REPORT = 10,
+	ASPA = 11,
 };
 
 #define FLAG_ANNOUNCE	0x1
@@ -73,6 +76,16 @@ struct rtr_ipv6 {
 	uint32_t	asnum;
 };
 
+#define FLAG_AFI_V6	0x1
+#define FLAG_AFI_MASK	FLAG_AFI_V6
+struct rtr_aspa {
+	uint8_t		flags;
+	uint8_t		afi_flags;
+	uint16_t	cnt;
+	uint32_t	cas;
+	/* array of spas with cnt elements follows */
+};
+
 struct rtr_endofdata {
 	uint32_t	serial;
 	uint32_t	refresh;
@@ -83,16 +96,19 @@ struct rtr_endofdata {
 enum rtr_event {
 	RTR_EVNT_START,
 	RTR_EVNT_CON_OPEN,
-	RTR_EVNT_CON_CLOSED,
+	RTR_EVNT_CON_CLOSE,
 	RTR_EVNT_TIMER_REFRESH,
 	RTR_EVNT_TIMER_RETRY,
 	RTR_EVNT_TIMER_EXPIRE,
+	RTR_EVNT_TIMER_ACTIVE,
 	RTR_EVNT_SEND_ERROR,
 	RTR_EVNT_SERIAL_NOTIFY,
 	RTR_EVNT_CACHE_RESPONSE,
 	RTR_EVNT_END_OF_DATA,
 	RTR_EVNT_CACHE_RESET,
 	RTR_EVNT_NO_DATA,
+	RTR_EVNT_RESET_AND_CLOSE,
+	RTR_EVNT_UNSUPP_PROTO_VERSION,
 };
 
 static const char *rtr_eventnames[] = {
@@ -102,32 +118,40 @@ static const char *rtr_eventnames[] = {
 	"refresh timer expired",
 	"retry timer expired",
 	"expire timer expired",
+	"activity timer expired",
 	"sent error",
 	"serial notify received",
 	"cache response received",
 	"end of data received",
 	"cache reset received",
-	"no data"
+	"no data",
+	"connection closed with reset",
+	"unsupported protocol version",
 };
 
 enum rtr_state {
 	RTR_STATE_CLOSED,
 	RTR_STATE_ERROR,
+	/* sessions with a state below this line will poll for incoming data */
 	RTR_STATE_IDLE,
 	RTR_STATE_ACTIVE,
+	RTR_STATE_NEGOTIATION,
 };
 
 static const char *rtr_statenames[] = {
 	"closed",
 	"error",
 	"idle",
-	"active"
+	"active",
+	"negotiation",
 };
 
 struct rtr_session {
 	TAILQ_ENTRY(rtr_session)	entry;
 	char				descr[PEER_DESCR_LEN];
 	struct roa_tree			roa_set;
+	struct aspa_tree		aspa_v4;
+	struct aspa_tree		aspa_v6;
 	struct ibuf_read		r;
 	struct msgbuf			w;
 	struct timer_head		timers;
@@ -136,14 +160,17 @@ struct rtr_session {
 	uint32_t			refresh;
 	uint32_t			retry;
 	uint32_t			expire;
+	uint32_t			active;
 	int				session_id;
 	int				fd;
+	int				active_lock;
 	enum rtr_state			state;
 	enum reconf_action		reconf_action;
 	enum rtr_error			last_sent_error;
 	enum rtr_error			last_recv_error;
 	char				last_sent_msg[REASON_LEN];
 	char				last_recv_msg[REASON_LEN];
+	uint8_t				version;
 };
 
 TAILQ_HEAD(, rtr_session) rtrs = TAILQ_HEAD_INITIALIZER(rtrs);
@@ -182,14 +209,28 @@ log_rtr_type(enum rtr_pdu_type type)
 		return "router key";
 	case ERROR_REPORT:
 		return "error report";
+	case ASPA:
+		return "aspa pdu";
 	default:
 		snprintf(buf, sizeof(buf), "unknown %u", type);
 		return buf;
 	}
 };
 
+static void
+rtr_reset_cache(struct rtr_session *rs)
+{
+	/* reset session */
+	rs->session_id = -1;
+	timer_stop(&rs->timers, Timer_Rtr_Expire);
+	free_roatree(&rs->roa_set);
+	free_aspatree(&rs->aspa_v4);
+	free_aspatree(&rs->aspa_v6);
+}
+
 static struct ibuf *
-rtr_newmsg(enum rtr_pdu_type type, uint32_t len, uint16_t session_id)
+rtr_newmsg(struct rtr_session *rs, enum rtr_pdu_type type, uint32_t len,
+    uint16_t session_id)
 {
 	struct ibuf *buf;
 	struct rtr_header rh;
@@ -203,7 +244,7 @@ rtr_newmsg(enum rtr_pdu_type type, uint32_t len, uint16_t session_id)
 		return NULL;
 
 	memset(&rh, 0, sizeof(rh));
-	rh.version = 1;
+	rh.version = rs->version;
 	rh.type = type;
 	rh.session_id = htons(session_id);
 	rh.length = htonl(len);
@@ -232,9 +273,8 @@ rtr_send_error(struct rtr_session *rs, enum rtr_error err, char *msg,
 	} else
 		memset(rs->last_sent_msg, 0, sizeof(rs->last_sent_msg));
 
-	rtr_fsm(rs, RTR_EVNT_SEND_ERROR);
-
-	buf = rtr_newmsg(ERROR_REPORT, 2 * sizeof(hdrlen) + len + mlen, err);
+	buf = rtr_newmsg(rs, ERROR_REPORT, 2 * sizeof(hdrlen) + len + mlen,
+	    err);
 	if (buf == NULL) {
 		log_warn("rtr %s: send error report", log_rtr(rs));
 		return;
@@ -249,16 +289,18 @@ rtr_send_error(struct rtr_session *rs, enum rtr_error err, char *msg,
 	ibuf_add(buf, msg, mlen);
 	ibuf_close(&rs->w, buf);
 
-	log_warnx("%s: sending error report[%u] %s", log_rtr(rs), err,
+	log_warnx("rtr %s: sending error report[%u] %s", log_rtr(rs), err,
 	    msg ? msg : "");
+
+	rtr_fsm(rs, RTR_EVNT_SEND_ERROR);
 }
 
 static void
-rtr_reset_query(struct rtr_session *rs)
+rtr_send_reset_query(struct rtr_session *rs)
 {
 	struct ibuf *buf;
 
-	buf = rtr_newmsg(RESET_QUERY, 0, 0);
+	buf = rtr_newmsg(rs, RESET_QUERY, 0, 0);
 	if (buf == NULL) {
 		log_warn("rtr %s: send reset query", log_rtr(rs));
 		rtr_send_error(rs, INTERNAL_ERROR, "out of memory", NULL, 0);
@@ -268,12 +310,12 @@ rtr_reset_query(struct rtr_session *rs)
 }
 
 static void
-rtr_serial_query(struct rtr_session *rs)
+rtr_send_serial_query(struct rtr_session *rs)
 {
 	struct ibuf *buf;
 	uint32_t s;
 
-	buf = rtr_newmsg(SERIAL_QUERY, sizeof(s), rs->session_id);
+	buf = rtr_newmsg(rs, SERIAL_QUERY, sizeof(s), rs->session_id);
 	if (buf == NULL) {
 		log_warn("rtr %s: send serial query", log_rtr(rs));
 		rtr_send_error(rs, INTERNAL_ERROR, "out of memory", NULL, 0);
@@ -302,9 +344,10 @@ rtr_parse_header(struct rtr_session *rs, void *buf,
 
 	memcpy(&rh, buf, sizeof(rh));
 
-	if (rh.version != 1) {
-		log_warnx("rtr %s: received message with unsupported version",
-		    log_rtr(rs));
+	if (rh.version != rs->version && rh.type != ERROR_REPORT) {
+ badversion:
+		log_warnx("rtr %s: received %s message: unexpected version %d",
+		    log_rtr(rs), log_rtr_type(rh.type), rh.version);
 		rtr_send_error(rs, UNEXP_PROTOCOL_VERS, NULL, &rh, sizeof(rh));
 		return -1;
 	}
@@ -312,7 +355,7 @@ rtr_parse_header(struct rtr_session *rs, void *buf,
 	*msgtype = rh.type;
 	*msglen = ntohl(rh.length);
 
-	switch (*msgtype) {
+	switch (rh.type) {
 	case SERIAL_NOTIFY:
 		session_id = rs->session_id;
 		len = 12;
@@ -341,39 +384,55 @@ rtr_parse_header(struct rtr_session *rs, void *buf,
 		len = 8;
 		break;
 	case ROUTER_KEY:
+		if (rs->version < 1)
+			goto badversion;
 		len = 36;	/* XXX probably too small, but we ignore it */
 		/* FALLTHROUGH */
 	case ERROR_REPORT:
 		if (*msglen > RTR_MAX_LEN) {
+ toobig:
 			log_warnx("rtr %s: received %s: msg too big: %zu byte",
-			    log_rtr(rs), log_rtr_type(*msgtype), *msglen);
+			    log_rtr(rs), log_rtr_type(rh.type), *msglen);
 			rtr_send_error(rs, CORRUPT_DATA, "too big",
 			    &rh, sizeof(rh));
 			return -1;
 		}
 		if (*msglen < len) {
+ toosmall:
 			log_warnx("rtr %s: received %s: msg too small: "
-			    "%zu byte", log_rtr(rs), log_rtr_type(*msgtype),
+			    "%zu byte", log_rtr(rs), log_rtr_type(rh.type),
 			    *msglen);
 			rtr_send_error(rs, CORRUPT_DATA, "too small",
 			    &rh, sizeof(rh));
 			return -1;
 		}
 		/*
-		 * session_id check ommitted since ROUTER_KEY and ERROR_REPORT
+		 * session_id check omitted since ROUTER_KEY and ERROR_REPORT
 		 * use the field for different things.
 		 */
 		return 0;
+	case ASPA:
+		if (rs->version < 2)
+			goto badversion;
+		session_id = 0;
+		/* unlike all other messages ASPA is variable sized */
+		if (*msglen > RTR_MAX_LEN)
+			goto toobig;
+		if (*msglen < sizeof(struct rtr_aspa))
+			goto toosmall;
+		/* len must be a multiple of 4 */
+		len = *msglen & ~0x3;
+		break;
 	default:
-		log_warnx("rtr %s: received unknown message: type %u",
-		    log_rtr(rs), *msgtype);
+		log_warnx("rtr %s: received unknown message: type %s",
+		    log_rtr(rs), log_rtr_type(rh.type));
 		rtr_send_error(rs, UNSUPP_PDU_TYPE, NULL, &rh, sizeof(rh));
 		return -1;
 	}
 
 	if (len != *msglen) {
 		log_warnx("rtr %s: received %s: illegal len: %zu byte not %u",
-		    log_rtr(rs), log_rtr_type(*msgtype), *msglen, len);
+		    log_rtr(rs), log_rtr_type(rh.type), *msglen, len);
 		rtr_send_error(rs, CORRUPT_DATA, "bad length",
 		    &rh, sizeof(rh));
 		return -1;
@@ -381,11 +440,11 @@ rtr_parse_header(struct rtr_session *rs, void *buf,
 
 	if (session_id != ntohs(rh.session_id)) {
 		/* ignore SERIAL_NOTIFY during startup */
-		if (rs->session_id == -1 && *msgtype == SERIAL_NOTIFY)
+		if (rs->session_id == -1 && rh.type == SERIAL_NOTIFY)
 			return 0;
 
 		log_warnx("rtr %s: received %s: bad session_id: %d != %d",
-		    log_rtr(rs), log_rtr_type(*msgtype), ntohs(rh.session_id),
+		    log_rtr(rs), log_rtr_type(rh.type), ntohs(rh.session_id),
 		    session_id);
 		rtr_send_error(rs, CORRUPT_DATA, "bad session_id",
 		    &rh, sizeof(rh));
@@ -398,9 +457,11 @@ rtr_parse_header(struct rtr_session *rs, void *buf,
 static int
 rtr_parse_notify(struct rtr_session *rs, uint8_t *buf, size_t len)
 {
-	if (rs->state == RTR_STATE_ACTIVE) {
-		log_warnx("rtr %s: received %s: while active (ignored)",
-		    log_rtr(rs), log_rtr_type(SERIAL_NOTIFY));
+	if (rs->state == RTR_STATE_ACTIVE ||
+	    rs->state == RTR_STATE_NEGOTIATION) {
+		log_warnx("rtr %s: received %s: while in state %s (ignored)",
+		    log_rtr(rs), log_rtr_type(SERIAL_NOTIFY),
+		    rtr_statenames[rs->state]);
 		return 0;
 	}
 
@@ -411,7 +472,7 @@ rtr_parse_notify(struct rtr_session *rs, uint8_t *buf, size_t len)
 static int
 rtr_parse_cache_response(struct rtr_session *rs, uint8_t *buf, size_t len)
 {
-	if (rs->state != RTR_STATE_IDLE) {
+	if (rs->state != RTR_STATE_IDLE && rs->state != RTR_STATE_NEGOTIATION) {
 		log_warnx("rtr %s: received %s: out of context",
 		    log_rtr(rs), log_rtr_type(CACHE_RESPONSE));
 		return -1;
@@ -476,7 +537,7 @@ rtr_parse_ipv4_prefix(struct rtr_session *rs, uint8_t *buf, size_t len)
 
 		r = RB_FIND(roa_tree, &rs->roa_set, roa);
 		if (r == NULL) {
-			log_warnx("rtr %s: received %s: unknown withdrawl",
+			log_warnx("rtr %s: received %s: unknown withdrawal",
 			    log_rtr(rs), log_rtr_type(IPV4_PREFIX));
 			rtr_send_error(rs, UNK_REC_WDRAWL, NULL, buf, len);
 			free(roa);
@@ -545,7 +606,7 @@ rtr_parse_ipv6_prefix(struct rtr_session *rs, uint8_t *buf, size_t len)
 
 		r = RB_FIND(roa_tree, &rs->roa_set, roa);
 		if (r == NULL) {
-			log_warnx("rtr %s: received %s: unknown withdrawl",
+			log_warnx("rtr %s: received %s: unknown withdrawal",
 			    log_rtr(rs), log_rtr_type(IPV6_PREFIX));
 			rtr_send_error(rs, UNK_REC_WDRAWL, NULL, buf, len);
 			free(roa);
@@ -558,6 +619,101 @@ rtr_parse_ipv6_prefix(struct rtr_session *rs, uint8_t *buf, size_t len)
 	return 0;
 }
 
+static int
+rtr_parse_aspa(struct rtr_session *rs, uint8_t *buf, size_t len)
+{
+	struct rtr_aspa rtr_aspa;
+	struct aspa_tree *aspatree;
+	struct aspa_set *aspa, *a;
+	size_t offset;
+	uint16_t cnt, i;
+	uint8_t aid;
+
+	memcpy(&rtr_aspa, buf + sizeof(struct rtr_header), sizeof(rtr_aspa));
+	offset = sizeof(struct rtr_header) + sizeof(rtr_aspa);
+	cnt = ntohs(rtr_aspa.cnt);
+	if (len != offset + cnt * sizeof(uint32_t)) {
+		log_warnx("rtr %s: received %s: bad pdu len",
+		    log_rtr(rs), log_rtr_type(ASPA));
+		rtr_send_error(rs, CORRUPT_DATA, "bad len", buf, len);
+		return -1;
+	}
+
+	if (rs->state != RTR_STATE_ACTIVE) {
+		log_warnx("rtr %s: received %s: out of context",
+		    log_rtr(rs), log_rtr_type(ASPA));
+		rtr_send_error(rs, CORRUPT_DATA, NULL, buf, len);
+		return -1;
+	}
+
+	if (rtr_aspa.afi_flags & FLAG_AFI_V6) {
+		aid = AID_INET6;
+		aspatree = &rs->aspa_v6;
+	} else {
+		aid = AID_INET;
+		aspatree = &rs->aspa_v4;
+	}
+
+	/* create aspa_set entry from the rtr aspa pdu */
+	if ((aspa = calloc(1, sizeof(*aspa))) == NULL) {
+		log_warn("rtr %s: received %s",
+		    log_rtr(rs), log_rtr_type(ASPA));
+		rtr_send_error(rs, INTERNAL_ERROR, "out of memory", NULL, 0);
+		return -1;
+	}
+	aspa->as = ntohl(rtr_aspa.cas);
+	aspa->num = cnt;
+	if (cnt > 0) {
+		if ((aspa->tas = calloc(cnt, sizeof(uint32_t))) == NULL ||
+		    (aspa->tas_aid = calloc(cnt, 1)) == NULL) {
+			free_aspa(aspa);
+			log_warn("rtr %s: received %s",
+			    log_rtr(rs), log_rtr_type(ASPA));
+			rtr_send_error(rs, INTERNAL_ERROR, "out of memory",
+			    NULL, 0);
+			return -1;
+		}
+		for (i = 0; i < cnt; i++) {
+			uint32_t tas;
+			memcpy(&tas, buf + offset + i * sizeof(tas),
+			    sizeof(tas));
+			aspa->tas[i] = ntohl(tas);
+			aspa->tas_aid[i] = aid;
+		}
+	}
+
+	if (rtr_aspa.flags & FLAG_ANNOUNCE) {
+		a = RB_INSERT(aspa_tree, aspatree, aspa);
+		if (a != NULL) {
+			RB_REMOVE(aspa_tree, aspatree, a);
+			free_aspa(a);
+
+			if (RB_INSERT(aspa_tree, aspatree, aspa) != NULL) {
+				log_warnx("rtr %s: received %s: corrupt tree",
+				    log_rtr(rs), log_rtr_type(ASPA));
+				rtr_send_error(rs, INTERNAL_ERROR,
+				    "corrupt aspa tree", NULL, 0);
+				free_aspa(aspa);
+				return -1;
+			}
+		}
+	} else {
+		a = RB_FIND(aspa_tree, aspatree, aspa);
+		if (a == NULL) {
+			log_warnx("rtr %s: received %s: unknown withdrawal",
+			    log_rtr(rs), log_rtr_type(ASPA));
+			rtr_send_error(rs, UNK_REC_WDRAWL, NULL, buf, len);
+			free_aspa(aspa);
+			return -1;
+		}
+		RB_REMOVE(aspa_tree, aspatree, a);
+		free_aspa(a);
+		free_aspa(aspa);
+	}
+
+	return 0;
+}
+
 static int
 rtr_parse_end_of_data(struct rtr_session *rs, uint8_t *buf, size_t len)
 {
@@ -573,14 +729,14 @@ rtr_parse_end_of_data(struct rtr_session *rs, uint8_t *buf, size_t len)
 		return -1;
 	}
 
-	memcpy(&eod, buf, sizeof(eod));
-
 	if (rs->state != RTR_STATE_ACTIVE) {
 		log_warnx("rtr %s: received %s: out of context",
 		    log_rtr(rs), log_rtr_type(END_OF_DATA));
 		return -1;
 	}
 
+	memcpy(&eod, buf, sizeof(eod));
+
 	rs->serial = ntohl(eod.serial);
 	/* validate timer values to be in the right range */
 	t = ntohl(eod.refresh);
@@ -640,12 +796,12 @@ rtr_parse_error(struct rtr_session *rs, uint8_t *buf, size_t len)
 	errcode = ntohs(rh.session_id);
 
 	memcpy(&pdu_len, buf, sizeof(pdu_len));
-	pdu_len = ntohs(pdu_len);
+	pdu_len = ntohl(pdu_len);
 
 	if (len < pdu_len + sizeof(pdu_len)) {
-		log_warnx("rtr %s: received %s: bad pdu len: %u byte",
-		    log_rtr(rs), log_rtr_type(ERROR_REPORT), pdu_len);
-		rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+		log_warnx("rtr %s: received %s: bad encapsulated pdu len: %u "
+		    "byte", log_rtr(rs), log_rtr_type(ERROR_REPORT), pdu_len);
+		rtr_fsm(rs, RTR_EVNT_RESET_AND_CLOSE);
 		return -1;
 	}
 
@@ -654,12 +810,12 @@ rtr_parse_error(struct rtr_session *rs, uint8_t *buf, size_t len)
 	len -= pdu_len + sizeof(pdu_len);
 
 	memcpy(&msg_len, buf, sizeof(msg_len));
-	msg_len = ntohs(msg_len);
+	msg_len = ntohl(msg_len);
 
 	if (len < msg_len + sizeof(msg_len)) {
 		log_warnx("rtr %s: received %s: bad msg len: %u byte",
 		    log_rtr(rs), log_rtr_type(ERROR_REPORT), msg_len);
-		rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+		rtr_fsm(rs, RTR_EVNT_RESET_AND_CLOSE);
 		return -1;
 	}
 
@@ -673,22 +829,23 @@ rtr_parse_error(struct rtr_session *rs, uint8_t *buf, size_t len)
 
 	if (errcode == NO_DATA_AVAILABLE) {
 		rtr_fsm(rs, RTR_EVNT_NO_DATA);
-	} else {
-		rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
-		rs->last_recv_error = errcode;
-		if (str)
-			strlcpy(rs->last_recv_msg, str,
-			    sizeof(rs->last_recv_msg));
-		else
-			memset(rs->last_recv_msg, 0,
-			    sizeof(rs->last_recv_msg));
-
 		free(str);
-		return -1;
+		return 0;
 	}
-	free(str);
+	if (errcode == UNSUPP_PROTOCOL_VERS)
+		rtr_fsm(rs, RTR_EVNT_UNSUPP_PROTO_VERSION);
+	else
+		rtr_fsm(rs, RTR_EVNT_RESET_AND_CLOSE);
+	rs->last_recv_error = errcode;
+	if (str)
+		strlcpy(rs->last_recv_msg, str,
+		    sizeof(rs->last_recv_msg));
+	else
+		memset(rs->last_recv_msg, 0,
+		    sizeof(rs->last_recv_msg));
 
-	return 0;
+	free(str);
+	return -1;
 }
 
 /*
@@ -765,6 +922,11 @@ rtr_process_msg(struct rtr_session *rs)
 				/* no need to send back an error */
 				return;
 			break;
+		case ASPA:
+			if (rtr_parse_aspa(rs, rptr, msglen) == -1) {
+				return;
+			}
+			break;
 		default:
 			log_warnx("rtr %s: received %s: unexpected pdu type",
 			    log_rtr(rs), log_rtr_type(msgtype));
@@ -788,8 +950,49 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 	enum rtr_state prev_state = rs->state;
 
 	switch (event) {
-	case RTR_EVNT_CON_CLOSED:
-		if (rs->state != RTR_STATE_CLOSED) {
+	case RTR_EVNT_UNSUPP_PROTO_VERSION:
+		if (rs->state == RTR_STATE_NEGOTIATION) {
+			if (rs->version > 0)
+				rs->version--;
+			else {
+				/*
+				 * can't downgrade anymore, fail connection
+				 * RFC requires to send the error with our
+				 * highest version number.
+				 */
+				rs->version = RTR_MAX_VERSION;
+				log_warnx("rtr %s: version negotiation failed",
+				    log_rtr(rs));
+				rtr_send_error(rs, UNSUPP_PROTOCOL_VERS,
+				    NULL, NULL, 0);
+				return;
+			}
+
+			if (rs->fd != -1) {
+				/* flush buffers */
+				msgbuf_clear(&rs->w);
+				rs->r.wpos = 0;
+				close(rs->fd);
+				rs->fd = -1;
+			}
+
+			/* retry connection with lower version */
+			timer_set(&rs->timers, Timer_Rtr_Retry, rs->retry);
+			rtr_imsg_compose(IMSG_SOCKET_CONN, rs->id, 0, NULL, 0);
+			break;
+		}
+		/* FALLTHROUGH */
+	case RTR_EVNT_RESET_AND_CLOSE:
+		rtr_reset_cache(rs);
+		rtr_recalc();
+		/* FALLTHROUGH */
+	case RTR_EVNT_CON_CLOSE:
+		if (rs->state == RTR_STATE_NEGOTIATION) {
+			/* consider any close event as a version failure. */
+			rtr_fsm(rs, RTR_EVNT_UNSUPP_PROTO_VERSION);
+			break;
+		}
+		if (rs->fd != -1) {
 			/* flush buffers */
 			msgbuf_clear(&rs->w);
 			rs->r.wpos = 0;
@@ -805,7 +1008,7 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 	case RTR_EVNT_TIMER_RETRY:
 		switch (rs->state) {
 		case RTR_STATE_ERROR:
-			rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 			return;
 		case RTR_STATE_CLOSED:
 			timer_set(&rs->timers, Timer_Rtr_Retry, rs->retry);
@@ -818,9 +1021,9 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 	case RTR_EVNT_CON_OPEN:
 		timer_stop(&rs->timers, Timer_Rtr_Retry);
 		if (rs->session_id == -1)
-			rtr_reset_query(rs);
+			rtr_send_reset_query(rs);
 		else
-			rtr_serial_query(rs);
+			rtr_send_serial_query(rs);
 		break;
 	case RTR_EVNT_SERIAL_NOTIFY:
 		/* schedule a refresh after a quick wait */
@@ -829,28 +1032,41 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 		break;
 	case RTR_EVNT_TIMER_REFRESH:
 		/* send serial query */
-		rtr_serial_query(rs);
+		rtr_send_serial_query(rs);
 		break;
 	case RTR_EVNT_TIMER_EXPIRE:
-		free_roatree(&rs->roa_set);
+		rtr_reset_cache(rs);
 		rtr_recalc();
 		break;
+	case RTR_EVNT_TIMER_ACTIVE:
+		log_warnx("rtr %s: activity timer fired", log_rtr(rs));
+		rtr_sem_release(rs->active_lock);
+		rtr_recalc();
+		rs->active_lock = 0;
+		break;
 	case RTR_EVNT_CACHE_RESPONSE:
 		rs->state = RTR_STATE_ACTIVE;
 		timer_stop(&rs->timers, Timer_Rtr_Refresh);
 		timer_stop(&rs->timers, Timer_Rtr_Retry);
+		timer_set(&rs->timers, Timer_Rtr_Active, rs->active);
+		/* prevent rtr_recalc from running while active */
+		rs->active_lock = 1;
+		rtr_sem_acquire(rs->active_lock);
 		break;
 	case RTR_EVNT_END_OF_DATA:
 		/* start refresh and expire timers */
 		timer_set(&rs->timers, Timer_Rtr_Refresh, rs->refresh);
 		timer_set(&rs->timers, Timer_Rtr_Expire, rs->expire);
+		timer_stop(&rs->timers, Timer_Rtr_Active);
 		rs->state = RTR_STATE_IDLE;
+		rtr_sem_release(rs->active_lock);
 		rtr_recalc();
+		rs->active_lock = 0;
 		break;
 	case RTR_EVNT_CACHE_RESET:
-		/* reset session and retry after a quick wait */
-		rs->session_id = -1;
-		free_roatree(&rs->roa_set);
+		rtr_reset_cache(rs);
+		rtr_recalc();
+		/* retry after a quick wait */
 		timer_set(&rs->timers, Timer_Rtr_Retry,
 		    arc4random_uniform(10));
 		break;
@@ -859,8 +1075,11 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 		timer_set(&rs->timers, Timer_Rtr_Retry, rs->retry);
 		/* stop refresh timer just to be sure */
 		timer_stop(&rs->timers, Timer_Rtr_Refresh);
+		rs->state = RTR_STATE_IDLE;
 		break;
 	case RTR_EVNT_SEND_ERROR:
+		rtr_reset_cache(rs);
+		rtr_recalc();
 		rs->state = RTR_STATE_ERROR;
 		/* flush receive buffer */
 		rs->r.wpos = 0;
@@ -883,39 +1102,37 @@ rtr_dispatch_msg(struct pollfd *pfd, struct rtr_session *rs)
 
 	if (pfd->revents & POLLHUP) {
 		log_warnx("rtr %s: Connection closed, hangup", log_rtr(rs));
-		rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+		rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 		return;
 	}
 	if (pfd->revents & (POLLERR|POLLNVAL)) {
 		log_warnx("rtr %s: Connection closed, error", log_rtr(rs));
-		rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+		rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 		return;
 	}
 	if (pfd->revents & POLLOUT && rs->w.queued) {
-		if ((error = ibuf_write(&rs->w)) <= 0 && errno != EAGAIN) {
-			if (error == 0)
-				log_warnx("rtr %s: Connection closed",
-				    log_rtr(rs));
-			else if (error == -1)
+		if ((error = ibuf_write(&rs->w)) == -1) {
+			if (errno != EAGAIN) {
 				log_warn("rtr %s: write error", log_rtr(rs));
-			rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
-			return;
+				rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
+			}
 		}
+		if (error == 0)
+			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 		if (rs->w.queued == 0 && rs->state == RTR_STATE_ERROR)
-			rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 	}
 	if (pfd->revents & POLLIN) {
 		if ((n = read(rs->fd, rs->r.buf + rs->r.wpos,
 		    sizeof(rs->r.buf) - rs->r.wpos)) == -1) {
 			if (errno != EINTR && errno != EAGAIN) {
 				log_warn("rtr %s: read error", log_rtr(rs));
-				rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+				rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 			}
 			return;
 		}
 		if (n == 0) {
-			log_warnx("rtr %s: Connection closed", log_rtr(rs));
-			rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 			return;
 		}
 		rs->r.wpos += n;
@@ -964,6 +1181,9 @@ rtr_check_events(struct pollfd *pfds, size_t npfds)
 			case Timer_Rtr_Expire:
 				rtr_fsm(rs, RTR_EVNT_TIMER_EXPIRE);
 				break;
+			case Timer_Rtr_Active:
+				rtr_fsm(rs, RTR_EVNT_TIMER_ACTIVE);
+				break;
 			default:
 				fatalx("King Bula lost in time");
 			}
@@ -1025,15 +1245,19 @@ rtr_new(uint32_t id, char *descr)
 		fatal("RTR session %s", descr);
 
 	RB_INIT(&rs->roa_set);
+	RB_INIT(&rs->aspa_v4);
+	RB_INIT(&rs->aspa_v6);
 	TAILQ_INIT(&rs->timers);
 	msgbuf_init(&rs->w);
 
 	strlcpy(rs->descr, descr, sizeof(rs->descr));
 	rs->id = id;
 	rs->session_id = -1;
+	rs->version = RTR_MAX_VERSION;
 	rs->refresh = RTR_DEFAULT_REFRESH;
 	rs->retry = RTR_DEFAULT_RETRY;
 	rs->expire = RTR_DEFAULT_EXPIRE;
+	rs->active = RTR_DEFAULT_ACTIVE;
 	rs->state = RTR_STATE_CLOSED;
 	rs->reconf_action = RECONF_REINIT;
 	rs->last_recv_error = NO_ERROR;
@@ -1066,24 +1290,26 @@ rtr_free(struct rtr_session *rs)
 	if (rs == NULL)
 		return;
 
-	rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+	rtr_reset_cache(rs);
+	rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 	timer_remove_all(&rs->timers);
-	free_roatree(&rs->roa_set);
 	free(rs);
 }
 
 void
 rtr_open(struct rtr_session *rs, int fd)
 {
-	if (rs->state != RTR_STATE_CLOSED) {
+	if (rs->state != RTR_STATE_CLOSED &&
+	    rs->state != RTR_STATE_NEGOTIATION) {
 		log_warnx("rtr %s: bad session state", log_rtr(rs));
-		rtr_fsm(rs, RTR_EVNT_CON_CLOSED);
+		rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 	}
 
-	log_debug("rtr %s: connection opened", log_rtr(rs));
+	if (rs->state == RTR_STATE_CLOSED)
+		rs->version = RTR_MAX_VERSION;
 
 	rs->fd = rs->w.fd = fd;
-	rs->state = RTR_STATE_IDLE;
+	rs->state = RTR_STATE_NEGOTIATION;
 	rtr_fsm(rs, RTR_EVNT_CON_OPEN);
 }
 
@@ -1122,7 +1348,21 @@ rtr_roa_merge(struct roa_tree *rt)
 
 	TAILQ_FOREACH(rs, &rtrs, entry) {
 		RB_FOREACH(roa, roa_tree, &rs->roa_set)
-			roa_insert(rt, roa);
+			rtr_roa_insert(rt, roa);
+	}
+}
+
+void
+rtr_aspa_merge(struct aspa_tree *at)
+{
+	struct rtr_session *rs;
+	struct aspa_set *aspa;
+
+	TAILQ_FOREACH(rs, &rtrs, entry) {
+		RB_FOREACH(aspa, aspa_tree, &rs->aspa_v4)
+			rtr_aspa_insert(at, aspa);
+		RB_FOREACH(aspa, aspa_tree, &rs->aspa_v6)
+			rtr_aspa_insert(at, aspa);
 	}
 }
 
@@ -1146,6 +1386,7 @@ rtr_show(struct rtr_session *rs, pid_t pid)
 	memset(&msg, 0, sizeof(msg));
 
 	/* descr, remote_addr, local_addr and remote_port set by parent */
+	msg.version = rs->version;
 	msg.serial = rs->serial;
 	msg.refresh = rs->refresh;
 	msg.retry = rs->retry;
