@@ -1,4 +1,4 @@
-/*	$OpenBSD: session.c,v 1.443 2023/04/20 12:53:27 claudio Exp $ */
+/*	$OpenBSD: session.c,v 1.445 2023/05/25 14:20:25 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004, 2005 Henning Brauer <henning@openbsd.org>
@@ -1199,20 +1199,15 @@ session_setup_socket(struct peer *p)
 		return (-1);
 	}
 
-	/* only increase bufsize (and thus window) if md5 or ipsec is in use */
-	if (p->conf.auth.method != AUTH_NONE) {
-		/* try to increase bufsize. no biggie if it fails */
-		bsize = 65535;
-		while (bsize > 8192 &&
-		    setsockopt(p->fd, SOL_SOCKET, SO_RCVBUF, &bsize,
-		    sizeof(bsize)) == -1 && errno != EINVAL)
-			bsize /= 2;
-		bsize = 65535;
-		while (bsize > 8192 &&
-		    setsockopt(p->fd, SOL_SOCKET, SO_SNDBUF, &bsize,
-		    sizeof(bsize)) == -1 && errno != EINVAL)
-			bsize /= 2;
-	}
+	/* limit bufsize. no biggie if it fails */
+	bsize = 65535;
+	while (bsize > 8192 && setsockopt(p->fd, SOL_SOCKET, SO_RCVBUF,
+	    &bsize, sizeof(bsize)) == -1 && errno != EINVAL)
+		bsize /= 2;
+	bsize = 65535;
+	while (bsize > 8192 && setsockopt(p->fd, SOL_SOCKET, SO_SNDBUF,
+	    &bsize, sizeof(bsize)) == -1 && errno != EINVAL)
+		bsize /= 2;
 
 	return (0);
 }
@@ -2013,6 +2008,8 @@ session_process_msg(struct peer *p)
 		}
 	}
 
+	if (p->rbuf == NULL)
+		return;
 	if (rpos < av) {
 		left = av - rpos;
 		memmove(&p->rbuf->buf, p->rbuf->buf + rpos, left);
