@@ -45,13 +45,20 @@
 #include "bgplgd.h"
 #include "http.h"
 
-#define TIMEOUT_DEFAULT		 30
 #ifndef	WWW_USER
 #define WWW_USER		 "www"
 #endif
 #ifndef	BGPLGD_USER
 #define BGPLGD_USER		 "_bgplgd"
 #endif
+#ifndef RUNSTATEDIR
+#define RUNSTATEDIR		"/var/run"
+#endif
+#ifndef WWWRUNSTATEDIR
+#define WWWRUNSTATEDIR		"/var/www/run"
+#endif
+
+#define TIMEOUT_DEFAULT		 30
 
 #define FCGI_CONTENT_SIZE	 65535
 #define FCGI_PADDING_SIZE	 255
@@ -230,11 +237,8 @@ struct timeval		kill_timeout = { 5, 0 };
 struct slowcgi_proc	slowcgi_proc;
 int			debug = 0;
 int			on = 1;
-char			*fcgi_socket = "/var/www/run/bgplgd.sock";
+char			*fcgi_socket = WWWRUNSTATEDIR "/bgplgd.sock";
 char			*bgpctlpath = "bgpctl";
-#ifndef RUNSTATEDIR
-#define RUNSTATEDIR "/var/run"
-#endif
 char			*bgpctlsock = RUNSTATEDIR "/bgpd.rsock";
 
 
